/* -- stars01.cc --
 * Author: Ioannis Tsiombikas 2005
 * uses the simple neural network implementation (neuron.h/cc)
 * to classify the stars into brown dwarfs and neutron stars.
 */
#include <iostream>
#include "neuron.h"

using namespace std;

#define DATA_COUNT		12
#define INPUT_COUNT		2
#define HIDDEN_COUNT	2

const char *class_name[] = {
	"brown dwarf",
	"neutron star"
};

// {mass, temperature} pairs.
float data[DATA_COUNT][2] = {
	{5, 4},
	{3, 9},
	{9, 7},
	{17, 4},
	{23, 5},
	{18, 9},
	{8, 13},
	{8, 21},
	{13, 25},
	{25, 18},
	{2, 23},
	{22, 29}
};

// for each hidden neuron, its two weights and a bias.
float w_hid[HIDDEN_COUNT][3] = {
	{2, 1, -28},
	{1, 1, -40}
};

// the weights & bias of the output neuron.
float w_out[3] = {1, -1, -0.5};


// initializes connections and weights.
void init_network(Neuron *in, Neuron *hid, Neuron *out) {
	for(int i=0; i<HIDDEN_COUNT; i++) {
		in[0].add_connection(&hid[i], w_hid[i][0]);
		in[1].add_connection(&hid[i], w_hid[i][1]);
		hid[i].set_bias(w_hid[i][2]);

		hid[i].add_connection(out, w_out[i]);
	}
	out->set_bias(w_out[2]);
}


int main(void) {
	// create the layers...
	InputNeuron input[DATA_COUNT];
	Neuron hidden[HIDDEN_COUNT];
	Neuron output;

	// connect them...
	init_network(input, hidden, &output);

	// and start processing...
	cout << " # | z1 | z2 | output\n---+----+----+---------\n";
	for(int i=0; i<DATA_COUNT; i++) {
		cout << (i < 9 ? " " : "") << i + 1;

		for(int j=0; j<INPUT_COUNT; j++) {
			input[j].input(data[i][j]);
			input[j].fire();
		}

		for(int j=0; j<HIDDEN_COUNT; j++) {
			cout << " |  " << hidden[j].fire();
		}

		int val = (int)output.fire();
		cout << " |  " << val << " (" << class_name[val & 1] << ")";
		cout << endl;
	}
}
