/* -- stars02.cc --
 * Author: Ioannis Tsiombikas 2005
 * uses the simple neural network implementation (neuron.h/cc)
 * to classify the stars into brown dwarfs, neutron stars, and
 * black holes (singularities).
 */
#include <iostream>
#include "neuron.h"

using namespace std;

#define DATA_COUNT		12
#define INPUT_COUNT		2
#define HIDDEN_COUNT	3
#define OUTPUT_COUNT	2

const char *class_name[] = {
	"brown dwarf",
	"-- can't happen --",
	"black hole",
	"neutron star"
};

// {mass, temperature} pairs.
float data[DATA_COUNT][2] = {
	{5, 4},
	{3, 9},
	{9, 7},
	{17, 4},
	{23, 5},
	{18, 9},
	{8, 13},
	{8, 21},
	{13, 25},
	{25, 18},
	{2, 23},
	{22, 29}
};

// for each hidden neuron, its two weights and a bias.
float w[HIDDEN_COUNT][3] = {
	{2, 1, -28},
	{1, 1, -40},
	{1, -1, -8}
};

// the weights & bias of the output neurons.
float w_out[OUTPUT_COUNT][4] = {
	{1, -1, 1, -0.5},
	{1, -1, -1, -0.5}
};
	


// initializes connections and weights.
void init_network(Neuron *in, Neuron *hid, Neuron *out) {
	for(int i=0; i<HIDDEN_COUNT; i++) {
		in[0].add_connection(&hid[i], w[i][0]);
		in[1].add_connection(&hid[i], w[i][1]);
		hid[i].set_bias(w[i][2]);

		hid[i].add_connection(&out[0], w_out[0][i]);
		hid[i].add_connection(&out[1], w_out[1][i]);
	}
	out->set_bias(w_out[0][3]);
	out->set_bias(w_out[1][3]);
}


int main(void) {
	// create the layers...
	InputNeuron input[DATA_COUNT];
	Neuron hidden[HIDDEN_COUNT];
	Neuron output[OUTPUT_COUNT];

	// connect them...
	init_network(input, hidden, output);

	// and start processing...
	cout << " # | z1 | z2 | z3 | output\n---+----+----+----+-------\n";
	for(int i=0; i<DATA_COUNT; i++) {
		cout << (i < 9 ? " " : "") << i + 1;

		for(int j=0; j<INPUT_COUNT; j++) {
			input[j].input(data[i][j]);
			input[j].fire();
		}

		for(int j=0; j<HIDDEN_COUNT; j++) {
			cout << " |  " << hidden[j].fire();
		}
		cout << " | ";

		int val[OUTPUT_COUNT];
		for(int j=0; j<OUTPUT_COUNT; j++) {
			val[j] = (int)output[j].fire();
			cout << " " << val[j];
		}
		
		int res = (val[0] << 1 | val[1]); // make the name array index
		cout << " (" << class_name[res & 3] << ")" << endl;
	}
}
